type Derivation<T> = (previous: T) => T;
type Accessor<T> = () => T;
type Setter<T> = (value: T) => void;
type Signal<T> = [Accessor<T>, Setter<T>];

declare function signal<T>(initial: T): Signal<T>;

declare function effect(fn: () => void): void;

declare function derive<T>(initial: T, fn: Derivation<T>): Accessor<T>;
declare function d(strings: TemplateStringsArray, ...values: (Accessor<any> | any)[]): Accessor<string>;

type HTMLTag = HTMLElementTagNameMap;
type Component<P> = ((props: P, children?: Children) => Element);
type HTMLComponent<P extends keyof HTMLTag> = Component<DoomProperties<P> | Children | undefined>;
type Children = Reactive<Child[] | Child>;
type Child = Element | Reactive<string>;
type Reactive<T> = T | Accessor<T>;
type IfEquals<X, Y, A, B> = (<T>() => T extends X ? 1 : 2) extends (<T>() => T extends Y ? 1 : 2) ? A : B;
type WritableKeysOf<T> = {
    [P in keyof T]: IfEquals<{
        [Q in P]: T[P];
    }, {
        -readonly [Q in P]: T[P];
    }, P, never>;
}[keyof T];
type WritablePart<T> = Pick<T, WritableKeysOf<T>>;
type FunctionPropertyNames<T> = {
    [K in keyof T]: T[K] extends Function ? K : never;
}[keyof T];
type WritableCSSStyleDeclaration = WritablePart<CSSStyleDeclaration>;
type StylesDeclaration = Omit<WritableCSSStyleDeclaration, FunctionPropertyNames<WritableCSSStyleDeclaration>>;
type Styles = {
    [K in keyof StylesDeclaration as K extends keyof StylesDeclaration ? K : never]?: Reactive<StylesDeclaration[K]>;
};
type PartialWithStyles<T> = Partial<T & {
    style: Reactive<Styles>;
}>;
type ElementWithoutEvents<T extends keyof HTMLTag> = Omit<HTMLTag[T], keyof GlobalEventHandlers | FunctionPropertyNames<Element> | 'innerHTML' | 'innerText' | 'outerHTML' | 'outerText'>;
type ElementProperties<T extends keyof HTMLTag> = PartialWithStyles<WritablePart<ElementWithoutEvents<T>>>;
type ElementEvents = Partial<Omit<GlobalEventHandlers, FunctionPropertyNames<Element>>>;
type DoomProperties<T extends keyof HTMLTag> = {
    [K in keyof ElementProperties<T> as K extends keyof HTMLTag[T] ? K : never]?: Reactive<ElementProperties<T>[K]>;
} & ElementEvents;

declare function h<K extends keyof HTMLTag>(component: K, a?: DoomProperties<K> | Children, b?: Children): Element;

declare const A: HTMLComponent<"a">;
declare const Abbr: HTMLComponent<"abbr">;
declare const Address: HTMLComponent<"address">;
declare const Area: HTMLComponent<"area">;
declare const Article: HTMLComponent<"article">;
declare const Aside: HTMLComponent<"aside">;
declare const Audio: HTMLComponent<"audio">;
declare const B: HTMLComponent<"b">;
declare const Base: HTMLComponent<"base">;
declare const Bdi: HTMLComponent<"bdi">;
declare const Bdo: HTMLComponent<"bdo">;
declare const Blockquote: HTMLComponent<"blockquote">;
declare const Body: HTMLComponent<"body">;
declare const Br: HTMLComponent<"br">;
declare const Button: HTMLComponent<"button">;
declare const Canvas: HTMLComponent<"canvas">;
declare const Caption: HTMLComponent<"caption">;
declare const Cite: HTMLComponent<"cite">;
declare const Code: HTMLComponent<"code">;
declare const Col: HTMLComponent<"col">;
declare const Colgroup: HTMLComponent<"colgroup">;
declare const Data: HTMLComponent<"data">;
declare const Datalist: HTMLComponent<"datalist">;
declare const Dd: HTMLComponent<"dd">;
declare const Del: HTMLComponent<"del">;
declare const Details: HTMLComponent<"details">;
declare const Dfn: HTMLComponent<"dfn">;
declare const Dialog: HTMLComponent<"dialog">;
declare const Div: HTMLComponent<"div">;
declare const Dl: HTMLComponent<"dl">;
declare const Dt: HTMLComponent<"dt">;
declare const Em: HTMLComponent<"em">;
declare const Embed: HTMLComponent<"embed">;
declare const Fieldset: HTMLComponent<"fieldset">;
declare const Figcaption: HTMLComponent<"figcaption">;
declare const Figure: HTMLComponent<"figure">;
declare const Footer: HTMLComponent<"footer">;
declare const Form: HTMLComponent<"form">;
declare const H1: HTMLComponent<"h1">;
declare const H2: HTMLComponent<"h2">;
declare const H3: HTMLComponent<"h3">;
declare const H4: HTMLComponent<"h4">;
declare const H5: HTMLComponent<"h5">;
declare const H6: HTMLComponent<"h6">;
declare const Head: HTMLComponent<"head">;
declare const Header: HTMLComponent<"header">;
declare const Hgroup: HTMLComponent<"hgroup">;
declare const Hr: HTMLComponent<"hr">;
declare const Html: HTMLComponent<"html">;
declare const I: HTMLComponent<"i">;
declare const Iframe: HTMLComponent<"iframe">;
declare const Img: HTMLComponent<"img">;
declare const Input: HTMLComponent<"input">;
declare const Ins: HTMLComponent<"ins">;
declare const Kbd: HTMLComponent<"kbd">;
declare const Label: HTMLComponent<"label">;
declare const Legend: HTMLComponent<"legend">;
declare const Li: HTMLComponent<"li">;
declare const Link: HTMLComponent<"link">;
declare const Main: HTMLComponent<"main">;
declare const Map: HTMLComponent<"map">;
declare const Mark: HTMLComponent<"mark">;
declare const Menu: HTMLComponent<"menu">;
declare const Meta: HTMLComponent<"meta">;
declare const Meter: HTMLComponent<"meter">;
declare const Nav: HTMLComponent<"nav">;
declare const Noscript: HTMLComponent<"noscript">;
declare const Ol: HTMLComponent<"ol">;
declare const Optgroup: HTMLComponent<"optgroup">;
declare const Option: HTMLComponent<"option">;
declare const Output: HTMLComponent<"output">;
declare const P: HTMLComponent<"p">;
declare const Picture: HTMLComponent<"picture">;
declare const Pre: HTMLComponent<"pre">;
declare const Progress: HTMLComponent<"progress">;
declare const Q: HTMLComponent<"q">;
declare const Rp: HTMLComponent<"rp">;
declare const Rt: HTMLComponent<"rt">;
declare const Ruby: HTMLComponent<"ruby">;
declare const S: HTMLComponent<"s">;
declare const Samp: HTMLComponent<"samp">;
declare const Script: HTMLComponent<"script">;
declare const Section: HTMLComponent<"section">;
declare const Select: HTMLComponent<"select">;
declare const Slot: HTMLComponent<"slot">;
declare const Small: HTMLComponent<"small">;
declare const Source: HTMLComponent<"source">;
declare const Span: HTMLComponent<"span">;
declare const Strong: HTMLComponent<"strong">;
declare const Style: HTMLComponent<"style">;
declare const Sub: HTMLComponent<"sub">;
declare const Summary: HTMLComponent<"summary">;
declare const Sup: HTMLComponent<"sup">;
declare const Table: HTMLComponent<"table">;
declare const Tbody: HTMLComponent<"tbody">;
declare const Td: HTMLComponent<"td">;
declare const Template: HTMLComponent<"template">;
declare const Textarea: HTMLComponent<"textarea">;
declare const Tfoot: HTMLComponent<"tfoot">;
declare const Th: HTMLComponent<"th">;
declare const Thead: HTMLComponent<"thead">;
declare const Time: HTMLComponent<"time">;
declare const Title: HTMLComponent<"title">;
declare const Tr: HTMLComponent<"tr">;
declare const Track: HTMLComponent<"track">;
declare const U: HTMLComponent<"u">;
declare const Ul: HTMLComponent<"ul">;
declare const Var: HTMLComponent<"var">;
declare const Video: HTMLComponent<"video">;
declare const Wbr: HTMLComponent<"wbr">;

export { A, Abbr, Address, Area, Article, Aside, Audio, B, Base, Bdi, Bdo, Blockquote, Body, Br, Button, Canvas, Caption, Cite, Code, Col, Colgroup, Data, Datalist, Dd, Del, Details, Dfn, Dialog, Div, Dl, Dt, Em, Embed, Fieldset, Figcaption, Figure, Footer, Form, H1, H2, H3, H4, H5, H6, Head, Header, Hgroup, Hr, Html, I, Iframe, Img, Input, Ins, Kbd, Label, Legend, Li, Link, Main, Map, Mark, Menu, Meta, Meter, Nav, Noscript, Ol, Optgroup, Option, Output, P, Picture, Pre, Progress, Q, Rp, Rt, Ruby, S, Samp, Script, Section, Select, Slot, Small, Source, Span, Strong, Style, Sub, Summary, Sup, Table, Tbody, Td, Template, Textarea, Tfoot, Th, Thead, Time, Title, Tr, Track, U, Ul, Var, Video, Wbr, d, derive, effect, h, signal };
